#=======================================================================
#	 ____  ____  _____              _    ____ ___   ____
#	|  _ \|  _ \|  ___|  _   _     / \  |  _ \_ _| |___ \
#	| |_) | | | | |_    (_) (_)   / _ \ | |_) | |    __) |
#	|  __/| |_| |  _|    _   _   / ___ \|  __/| |   / __/
#	|_|   |____/|_|     (_) (_) /_/   \_\_|  |___| |_____|
#
#	Copyright 1999-2001 Alfred Reibenschuh <areibens@cpan.org>.
#
#	This library is free software; you can redistribute it 
#	and/or modify it under the same terms as Perl itself.
#
#=======================================================================

$fonts->{'symbol'} = {
	'apiname' => 'Sym',
	'ascender' => 600,
	'capheight' => 600,
	'char' => [undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,'space','exclam','universal','numbersign','existential','percent','ampersand','suchthat','parenleft','parenright','asteriskmath','plus','comma','minus','period','slash','zero','one','two','three','four','five','six','seven','eight','nine','colon','semicolon','less','equal','greater','question','congruent','Alpha','Beta','Chi','Delta','Epsilon','Phi','Gamma','Eta','Iota','theta1','Kappa','Lambda','Mu','Nu','Omicron','Pi','Theta','Rho','Sigma','Tau','Upsilon','sigma1','Omega','Xi','Psi','Zeta','bracketleft','therefore','bracketright','perpendicular','underscore','radicalex','alpha','beta','chi','delta','epsilon','phi','gamma','eta','iota','phi1','kappa','lambda','mu','nu','omicron','pi','theta','rho','sigma','tau','upsilon','omega1','omega','xi','psi','zeta','braceleft','bar','braceright','similar',undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,'Euro','Upsilon1','minute','lessequal','fraction','infinity','florin','club','diamond','heart','spade','arrowboth','arrowleft','arrowup','arrowright','arrowdown','degree','plusminus','second','greaterequal','multiply','proportional','partialdiff','bullet','divide','notequal','equivalence','approxequal','ellipsis','arrowvertex','arrowhorizex','carriagereturn','aleph','Ifraktur','Rfraktur','weierstrass','circlemultiply','circleplus','emptyset','intersection','union','propersuperset','reflexsuperset','notsubset','propersubset','reflexsubset','element','notelement','angle','gradient','registerserif','copyrightserif','trademarkserif','product','radical','dotmath','logicalnot','logicaland','logicalor','arrowdblboth','arrowdblleft','arrowdblup','arrowdblright','arrowdbldown','lozenge','angleleft','registersans','copyrightsans','trademarksans','summation','parenlefttp','parenleftex','parenleftbt','bracketlefttp','bracketleftex','bracketleftbt','bracelefttp','braceleftmid','braceleftbt','braceex',undef,'angleright','integral','integraltp','integralex','integralbt','parenrighttp','parenrightex','parenrightbt','bracketrighttp','bracketrightex','bracketrightbt','bracerighttp','bracerightmid','bracerightbt'],
	'descender' => -200,
	'fontbbox' => ['-180','-293','1090','1010'],
	'fontname' => 'Symbol',
	'iscore' => 1,
	'isfixedpitch' => 0,
	'issymbol' => 1,
	'italicangle' => '0',
	'stdhw' => '92',
	'stdvw' => '85',
	'type' => 'Type1',
	'underlineposition' => '-100',
	'underlinethickness' => '50',
	'wx' => {'integralbt' => '686','apple' => '790','ampersand' => '778','arrowdblright' => '987','exclam' => '333','psi' => '686','arrowdbldown' => '603','Epsilon' => '611','Pi' => '768','four' => '500','parenrightbt' => '384','arrowdblleft' => '987','fraction' => '167','arrowdblboth' => '1042','parenleft' => '333','lozenge' => '494','dotmath' => '250','trademarkserif' => '890','nu' => '521','zero' => '500','gamma' => '411','Kappa' => '722','Xi' => '645','.notdef' => 0,'sigma' => '603','two' => '500','phi1' => '603','iota' => '329','registersans' => '790','multiply' => '549','ellipsis' => '1000','Chi' => '722','arrowdblup' => '603','Theta' => '741','diamond' => '753','Phi' => '763','lessequal' => '549','aleph' => '823','tau' => '439','bracelefttp' => '494','bracerightmid' => '494','bar' => '200','lambda' => '549','Delta' => '612','florin' => '500','perpendicular' => '658','braceright' => '480','infinity' => '713','zeta' => '494','logicalor' => '603','similar' => '549','pi' => '549','braceleft' => '480','braceex' => '494','integral' => '274','propersuperset' => '713','second' => '411','circlemultiply' => '768','Upsilon1' => '620','equal' => '549','bracerightbt' => '494','copyrightserif' => '790','minus' => '549','semicolon' => '278','upsilon' => '576','rho' => '549','integraltp' => '686','plusminus' => '549','xi' => '493','underscore' => '500','proportional' => '713','alpha' => '631','bracketrightbt' => '384','carriagereturn' => '658','omega1' => '713','reflexsubset' => '713','integralex' => '686','Beta' => '667','parenrighttp' => '384','bracketright' => '333','notequal' => '549','parenleftbt' => '384','trademarksans' => '786','Gamma' => '603','parenrightex' => '384','arrowdown' => '603','Upsilon' => '690','Eta' => '722','arrowup' => '603','arrowleft' => '987','Lambda' => '686','Sigma' => '592','less' => '549','heart' => '753','arrowboth' => '1042','therefore' => '863','bracketleftbt' => '384','radical' => '549','Rfraktur' => '795','club' => '753','angleright' => '329','percent' => '833','chi' => '549','omega' => '686','plus' => '549','phi' => '521','minute' => '247','approxequal' => '549','theta1' => '631','summation' => '713','greaterequal' => '549','union' => '768','asteriskmath' => '500','existential' => '549','reflexsuperset' => '713','Iota' => '333','angleleft' => '329','Psi' => '795','arrowright' => '987','colon' => '278','arrowhorizex' => '1000','Euro' => '750','Mu' => '889','kappa' => '549','bracerighttp' => '494','emptyset' => '823','comma' => '250','logicaland' => '603','Ifraktur' => '686','arrowvertex' => '603','weierstrass' => '987','Alpha' => '722','eight' => '500','bracketrighttp' => '384','sigma1' => '439','omicron' => '549','congruent' => '549','question' => '444','Zeta' => '611','theta' => '521','gradient' => '713','element' => '713','bracketrightex' => '384','five' => '500','radicalex' => '500','Nu' => '722','parenlefttp' => '384','divide' => '549','three' => '500','notsubset' => '713','numbersign' => '500','Tau' => '611','bracketleft' => '333','space' => '250','braceleftbt' => '494','copyrightsans' => '790','seven' => '500','suchthat' => '439','partialdiff' => '494','parenleftex' => '384','nine' => '500','eta' => '603','delta' => '494','degree' => '400','product' => '823','Omicron' => '722','parenright' => '333','greater' => '549','bracketlefttp' => '384','notelement' => '713','Omega' => '768','beta' => '549','six' => '500','epsilon' => '439','braceleftmid' => '494','equivalence' => '549','slash' => '278','period' => '250','registerserif' => '790','bracketleftex' => '384','propersubset' => '713','universal' => '713','mu' => '576','circleplus' => '768','one' => '500','logicalnot' => '713','bullet' => '460','intersection' => '768','Rho' => '556','angle' => '768','spade' => '753'},
	'xheight' => 450,
};

1;
